%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get constants based on equivalent kernels for uniform kernel
%   See Fand and Gijbels (1996, p.64, 67)
%
% Imput:
%  v: v-th derivative of interest (strictly less than p)
%  p: order of polynomial for LPR (less than 6)
%
% Output
% Cvp: Constant based on
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [C_vp] = Const_LPR_U(v,p)

% S = (\mu_{j+k}), 0<=j,k<=p, defined in FG(96,p61), p=6
S5 = [1/2 1/4 1/6 1/8 1/10;
       1/4 1/6 1/8 1/10 1/12;
       1/6 1/8 1/10 1/12 1/14;
       1/8 1/10 1/12 1/14 1/16;
       1/10 1/12 1/14 1/16 1/18];
% Sstar = (\nu_{j+k}), 0<=j,k<=p, defined in FG(96,p61), p=6
K5 = [1/4 1/8 1/12 1/16 1/20;
        1/8 1/12 1/16 1/20 1/24;
        1/12 1/16 1/20 1/24 1/28;
        1/16 1/20 1/24 1/28 1/32;
        1/20 1/24 1/28 1/32 1/36];
        
% Vector of \mu_j,
% [\mu_1, \mu_2, ..., \mu_{13}]
mu_vec = [1/4 1/6 1/8 1/10 1/12 1/14 1/16 1/18 1/20];

Sinv_p = invpd(S5(1:p+1,1:p+1));

% int t^{p+1}K_{v,p}^*(t)dt
int_tp_K = Sinv_p(v+1,:)*mu_vec(1,(p+1):(2*p+1))';

% \int_K_{v,p}^{*2}(t)dt
int_K2 = Sinv_p(v+1,:)*K5(1:p+1,1:p+1)*Sinv_p(v+1,:)';

C_vp = ((factorial(p+1)^2*(2*v+1)*int_K2)/(2*(p+1-v)*int_tp_K^2))^(1/(2*p+3));
   

