%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Return AI criterion function
%
% h          : bandwidthes
% f_pilot    : pilot density estimate 1x2 vector
% f1_pilot   : pilot derivative estimate 1x2 vector
% m2_pilot   : pilot 2nd derivative estimate of m 1x2 vector
% m3_pilot   : pilot 3rd derivative estimate of m 1x2 vector
% v_pilot    : pilot variance estimate of m 1x2 vector
% Num        : # of observations [Num_p Num_m]
%               Num_p: # of observations in positive area
%
% Output
% Qn: criterion function as a function of theta1, theta2, and lambda
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Qn] = Qn_AI_h_RDD(h,f_pilot,f1_pilot,m2_pilot,m3_pilot,v_pilot,n)

% matrix for constants depending on kernel
% [b1 c1 c2 v]
k_const = [-0.1 -0.1 -0.08 24/5]; %[b_1 xi_1 xi_2 v]
b2 = zeros(1,2);
m2foverf12 = zeros(1,2);
for j=1:2
    m2foverf12(j) = m2_pilot(j)*f1_pilot/f_pilot/2;
    b2(j) = (-1)^(j+1)*(k_const(2)*(m2foverf12(j) + m3_pilot(j)/6)...   
         - k_const(3)*m2foverf12(j));
end
    
Qn = k_const(1)^2*(m2_pilot(1)*h(1)^2 - m2_pilot(2)*h(2)^2)^2/4 ...
+(b2(1)*h(1)^3 - b2(2)*h(2)^3)^2 ...
+ k_const(4)*(v_pilot(1)/h(1) + v_pilot(2)/h(2))/f_pilot/n;





