%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data generation
% 
% Input
% N: Sample size
% D: Simulation Designs (See Arai and Ichimura (2014, Section 3))
%
% Output
% Y: Outcome, 
% X: Forcing variable,
% Tmat: matrix of true parameter values
%   [true_f true_f1 true_sigma_p true_sigma_m
%       true_tau true_m21 true_m22 true_m31 true_m32 true_delta_p true_delta_m]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Y,X,Tmat] = data_RDD(N,D)

Y = zeros(N,1);
Const = ones(N,1);
Z = 2.*betarnd(2,4,N,1)-Const;
true_f = 0.625;
true_f1 = -1.25;

error0 = 0.1295*randn(N,1);
error = [error0 error0];
true_sigma_p = 0.1295^2;
true_sigma_m = 0.1295^2;

X = Z;
X2 = X.^2;
X3 = X2.*X;
X_mat = [Const X X2 X3 X2.^2 X2.*X3];

if D == 1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Design I (DGP I of IK)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Coef = [0.48 1.27 7.18 20.21 21.54 7.33;
        0.52 0.84 -3 7.99 -9.01 3.56]';
YY = X_mat*Coef+ error;

true_m21 = -3;
true_m22 = 7.18;
true_m31 = 7.99;
true_m32 = 20.21;
true_m41 = -9.01;
true_m42 = 21.54;
true_m51 = 3.56;
true_m52 = 7.33;
true_delta_p = 0.52;
true_delta_m = 0.48;
true_tau = true_delta_p - true_delta_m;

for j = 1:N
    if X(j,1) < 0
        Y(j,1) = YY(j,1);
    else
        Y(j,1) = YY(j,2);
    end
end

Tmat = [true_f true_f1 true_sigma_p true_sigma_m...
    true_tau true_m21 true_m22 true_m31 true_m32...
    true_delta_p true_delta_m true_m41 true_m42 true_m51 true_m52];

elseif D == 2
%%%%%%%%%%%
% Design II
%%%%%%%%%%%

Coef = [3.70 2.99 3.28 1.45 0.22 0.03;
    0.26 18.49 -54.80 74.30 -45.02 9.83]';
YY = X_mat*Coef+ error;

true_m21 = -54.80;
true_m22 = 3.28;
true_m31 = 74.30;
true_m32 = 1.45;
true_m41 = -45.02;
true_m42 = 0.22;
true_m51 = 9.83;
true_m52 = 0.03;
true_delta_p = 0.26;
true_delta_m = 3.70;
true_tau = true_delta_p - true_delta_m;

for j = 1:N
    if X(j,1) < 0
        Y(j,1) = YY(j,1);
    else
        Y(j,1) = YY(j,2);
    end
end

Tmat = [true_f true_f1 true_sigma_p true_sigma_m...
    true_tau true_m21 true_m22 true_m31 true_m32...
    true_delta_p true_delta_m true_m41 true_m42 true_m51 true_m52]; 




elseif D==3
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Design III (DGP III of IK)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
X_mat = [Const (X>=0) X X2 X3 X2.^2 X2.*X3];
Coef = [0.42 0.1 0.84 -3 7.99 -9.01 3.56]';
Y = X_mat*Coef + error0;

true_m21 = -3;
true_m22 = -3;
true_m31 = 7.99;
true_m32 = 7.99;
true_m41 = -9.01;
true_m42 = -9.01;
true_m51 = 3.56;
true_m52 = 3.56;
true_delta_p = 0.52;
true_delta_m = 0.42;
true_tau = true_delta_p - true_delta_m;

Tmat = [true_f true_f1 true_sigma_p true_sigma_m...
    true_tau true_m21 true_m22 true_m31 true_m32...
    true_delta_p true_delta_m true_m41 true_m42 true_m51 true_m52];

else
%%%%%%%%%%%%
% Design IV
%%%%%%%%%%%%
Coef = [0.03 -2.26 -13.14 -30.89 -31.98 -12.10;
        0.09 5.76 -42.56 120.90 -139.71 55.59]';
YY = X_mat*Coef+ error;

true_m21 = -42.56;
true_m22 = -13.14;
true_m31 = 120.9;
true_m32 = -30.89;
true_m41 = -139.71;
true_m42 = -31.98;
true_m51 = 55.59;
true_m52 = -12.1;
true_delta_p = 0.09;
true_delta_m = 0.03;
true_tau = true_delta_p - true_delta_m;

for j = 1:N
    if X(j,1) < 0
        Y(j,1) = YY(j,1);
    else
        Y(j,1) = YY(j,2);
    end
end

Tmat = [true_f true_f1 true_sigma_p true_sigma_m...
    true_tau true_m21 true_m22 true_m31 true_m32...
    true_delta_p true_delta_m true_m41 true_m42 true_m51 true_m52];  

end