function f1hat = density1d(X,y,k,h)

% Purpose: estimate 1st derivative density at a point
% --------------------------------------------
% Usage: results = density_est
%       X = a column vector of scalar random variables
%       y = point to estimate
%       k = choice of kernel function
%           k = 1, Gaussian
%           k = 2, Jones' with Epanechnikov
%       h = bandwidth
% --------------------------------------------

nobs = size(X,1);
A_h = (y-X)/h;
B_h = A_h.^2;

if k==1
    ExpA_h = sum((B_h-1).*exp(-B_h/2))/sqrt(2*pi);
else
    k2 =1/5;
    const = 3/4;
    ExpA_h = (-1)*const*sum(A_h.*(1-B_h).*(abs(A_h)<1))/k2;
end

f1hat = ExpA_h/nobs/h^2;
