%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Return Sharp RDD design estimator
%
% Imputs:
% Y      :DATA N*1
% X      :DATA N*1
% ep     :estimation point 1*1
% h_p    :bandwidth for plus
% h_m    :bandwidth for minus
%
% Outputs:
% tau    :RDD estimate
% delta_p:estimate for the right of the cut-off
% delta_m:estimate for the left of the cut-off
% Num: the number of observations used to estimate [right left]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [tau,delta_p,delta_m,Num] = srdd(Y,X,x_ep,h_p,h_m)

N=size(Y,1);
Xx = X - x_ep;
data_mat = [Y ones(N,1) Xx];
Xbin_bet_xh = (X>=x_ep).*(X<=x_ep+h_p);     % index for X in [x, x+h2_p]
Num_p = sum(Xbin_bet_xh);
data_trim_mat_p = data_mat(Xbin_bet_xh==1,:);
YT_p = data_trim_mat_p(:,1);
T_p = data_trim_mat_p(:,2:3);
Kvec_p = (1-abs(data_trim_mat_p(:,3))/h_p).*(abs(data_trim_mat_p(:,3))<h_p);
W_p = diag(Kvec_p);
lambda_p_all = (T_p'*W_p*T_p)\(T_p'*W_p*YT_p);
delta_p = lambda_p_all(1,1);

Xbin_bet_hx = (X<x_ep).*(X>=x_ep-h_m);     % index for X in [x, x+h2_p]
Num_m = sum(Xbin_bet_hx);
data_trim_mat_m = data_mat(Xbin_bet_hx==1,:);
YT_m = data_trim_mat_m(:,1);
T_m = data_trim_mat_m(:,2:3);
Kvec_m = (1-abs(data_trim_mat_m(:,3)/h_m)).*(abs(data_trim_mat_m(:,3))<h_m);
W_m = diag(Kvec_m);
lambda_m_all = (T_m'*W_m*T_m)\(T_m'*W_m*YT_m);
delta_m = lambda_m_all(1,1);

Num = [Num_p Num_m];
tau = delta_p - delta_m;
