%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulation for RD
%
% D: DGP design
% x: evaluation point
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear;

% Simulation design
M = 10000;         % # of iteration
n = 500;        % sample size
x_ep = 0;       % discontinuity point
D_num = 4;      % number of DGPs
methods = 4;   % number of bandwidth selection methods

% Functions used in this code
%   data_RDD.m

% Get constants based on equivalent kernel for plug-in bandwidth
 C23K_U = Const_LPR_U(2,3);
 C33K_U = Const_LPR_U(3,3);
 C34K_U = Const_LPR_U(3,4);

 C23K_T = Const_LPR(2,3);
 C33K_T = Const_LPR(3,3);
 C34K_T = Const_LPR(3,4);

 k_const = [-0.1 -0.1 -0.08 24/5]; % [b1 c1 c2 v]
            
% Initialization
h_mat_1 = zeros(M,1);
h_mat_2 = zeros(M,1);
h_mat_3 = zeros(M,1);
h_mat_4 = zeros(M,2);
h_mat_5 = zeros(M,1);

tau_bias_mat_1 = zeros(M,1);
tau_bias_mat_2 = zeros(M,1);
tau_bias_mat_3 = zeros(M,1);
tau_bias_mat_4 = zeros(M,1);
tau_bias_mat_5 = zeros(M,1);
tau_bias_mat_6 = zeros(M,1);
tau_bias_mat_7 = zeros(M,1);
tau_bias_mat_8 = zeros(M,1);

all_result_mat = zeros(methods*D_num,4);

tic

%
% Begin Monte Carlo
%

for D=1:4

rng(10);

if D==1
    load('SRD_h_D1_16_n500'); 
elseif D==2
    load('SRD_h_D2_16_n500'); 
elseif D==3
    load('SRD_h_D3_16_n500');  
else
    load('SRD_h_D4_16_n500'); 
end

    h_mat_aip = h_mat(:,1);
    h_mat_aim = h_mat(:,5);
    h_mat_indp = h_mat(:,2);
    h_mat_indm = h_mat(:,6);
    h_mat_ik09 = h_mat(:,3);
    h_mat_cct = h_mat(:,4);
    
    h_ai = [h_mat_aip h_mat_aim];
    h_ind = [h_mat_indp h_mat_indm];  
        
for ite = 1:M
    % Data generation
    [Y,X,Tmat] = data_RDD(n,D);
    true_f  = Tmat(1,1);
    true_f1 = Tmat(1,2);
    true_sigma_p = Tmat(1,3);
    true_sigma_m = Tmat(1,4);
    true_tau  = Tmat(1,5);
    true_m21 = Tmat(1,6);
    true_m22 = Tmat(1,7);
    true_m31 = Tmat(1,8);
    true_m32 = Tmat(1,9);
    true_delta_p = Tmat(1,10);
    true_delta_m = Tmat(1,11);
    true_m41 = Tmat(1,12);
    true_m42 = Tmat(1,13);
    true_m51 = Tmat(1,14);
    true_m52 = Tmat(1,15);
    m2_true = [true_m21 true_m22];
    m3_true = [true_m31 true_m32];
    delta_true = [true_delta_p true_delta_m];

 
%   
% 1. AI bandwidth (half)
%
   h_1 = h_ai(ite,:)/2;
   [tau,~,~,~] = srdd(Y,X,x_ep,h_1(1),h_1(2));
   tau_bias_mat_1(ite,1) = tau-true_tau;
   
%   
% 2. IND bandwidth (half)
%
    h_2 = h_ind(ite,:)/2;
    [tau,~,~,~] = srdd(Y,X,x_ep,h_2(1),h_2(2));
    tau_bias_mat_2(ite,1) = tau - true_tau;
    
%   
% 3. IK bandwidth (half)
%
    h_3 = h_mat_ik09(ite,1)/2;
    [tau,~,~,~] = srdd(Y,X,x_ep,h_3,h_3);
    tau_bias_mat_3(ite,1) = tau - true_tau;

%   
% 4. CCT bandwidth (half)
%
    h_4 = h_mat_cct(ite,1)/2;
    [tau,~,~,~] = srdd(Y,X,x_ep,h_4,h_4);
    tau_bias_mat_4(ite,1) = tau - true_tau;

%   
% 5. AI bandwidth (double)
%
   h_1 = h_ai(ite,:)*2;
   [tau,~,~,~] = srdd(Y,X,x_ep,h_1(1),h_1(2));
   tau_bias_mat_5(ite,1) = tau-true_tau;
   
%   
% 6. IND bandwidth (double)
%
    h_2 = h_ind(ite,:)*2;
    [tau,~,~,~] = srdd(Y,X,x_ep,h_2(1),h_2(2));
    tau_bias_mat_6(ite,1) = tau - true_tau;
    
%   
% 7. IK bandwidth (double)
%
    h_3 = h_mat_ik09(ite,1)*2;
    [tau,~,~,~] = srdd(Y,X,x_ep,h_3,h_3);
    tau_bias_mat_7(ite,1) = tau - true_tau;

%   
% 8. CCT bandwidth (double)
%
    h_4 = h_mat_cct(ite,1)*2;
    [tau,~,~,~] = srdd(Y,X,x_ep,h_4,h_4);
    tau_bias_mat_8(ite,1) = tau - true_tau;


%ite

if rem(ite,1000)==0
    ite
else
end
    
% end of M iteration
end


    trimper = 5;
    all_result_mat(4*(D-1)+1,1) = trimmean(tau_bias_mat_1,trimper);
    all_result_mat(4*(D-1)+2,1) = trimmean(tau_bias_mat_2,trimper);
    all_result_mat(4*(D-1)+3,1) = trimmean(tau_bias_mat_3,trimper);
    all_result_mat(4*(D-1)+4,1) = trimmean(tau_bias_mat_4,trimper); 
    all_result_mat(4*(D-1)+1,3) = trimmean(tau_bias_mat_5,trimper);
    all_result_mat(4*(D-1)+2,3) = trimmean(tau_bias_mat_6,trimper);
    all_result_mat(4*(D-1)+3,3) = trimmean(tau_bias_mat_7,trimper);
    all_result_mat(4*(D-1)+4,3) = trimmean(tau_bias_mat_8,trimper); 
    
    tau_bias1_absort = sort(tau_bias_mat_1);
    tau_bias2_absort = sort(tau_bias_mat_2);
    tau_bias3_absort = sort(tau_bias_mat_3);
    tau_bias4_absort = sort(tau_bias_mat_4);
    tau_bias5_absort = sort(tau_bias_mat_5);
    tau_bias6_absort = sort(tau_bias_mat_6);
    tau_bias7_absort = sort(tau_bias_mat_7);
    tau_bias8_absort = sort(tau_bias_mat_8);
    
    all_result_mat(4*(D-1)+1,2) = sqrt(mean(tau_bias1_absort(251:9750,:).^2));
    all_result_mat(4*(D-1)+2,2) = sqrt(mean(tau_bias2_absort(251:9750,:).^2));
    all_result_mat(4*(D-1)+3,2) = sqrt(mean(tau_bias3_absort(251:9750,:).^2));
    all_result_mat(4*(D-1)+4,2) = sqrt(mean(tau_bias4_absort(251:9750,:).^2));
    all_result_mat(4*(D-1)+1,4) = sqrt(mean(tau_bias5_absort(251:9750,:).^2));
    all_result_mat(4*(D-1)+2,4) = sqrt(mean(tau_bias6_absort(251:9750,:).^2));
    all_result_mat(4*(D-1)+3,4) = sqrt(mean(tau_bias7_absort(251:9750,:).^2));
    all_result_mat(4*(D-1)+4,4) = sqrt(mean(tau_bias8_absort(251:9750,:).^2));
    
D

% end of D loop
end

disp('Table 2 (n=500)')

all_result_mat

toc
beep
