function output = rddci_cct_b(X,Y,c,h_opt,b2)

%--------------------------------------------------------------------------
% rddci.m: compute the CI for sharp regression discontinuity estimator
%--------------------------------------------------------------------------
%

% Compute SRDD estimate
n = size(X,1);
Xx = X-c;
Xx2 = Xx.^2;
Xx3 = Xx.*Xx2;
Xx4 = Xx2.^2;
nones = ones(n,1);
data_mat = [Y nones Xx Xx2 Xx3 Xx4];
Xbin_lt_x = (X<c);           % binary indicator for observations less than x
Xbin_bt_x = (X>=c);          % bigger than x
data_mat_p = data_mat(Xbin_bt_x==1,:); % data_mat only for positive X
data_mat_m = data_mat(Xbin_lt_x==1,:); % data_mat only for negative X
yp = data_mat_p(:,1);
ym = data_mat_m(:,1);

% Compute tau and V1
e0 = [1 0]';
h_vec_p = [1 1/h_opt(1) 1/h_opt(1)^2];
h_vec_m = [1 1/h_opt(2) 1/h_opt(2)^2];
H_p2 = diag(h_vec_p(1:2));
H_m2 = diag(h_vec_m(1:2));
H_p3 = diag(h_vec_p);
H_m3 = diag(h_vec_m);
X3h_p = data_mat_p(:,2:4)*H_p3;
X3h_m = data_mat_m(:,2:4)*H_m3;
Kvec_h_p = (1-abs(data_mat_p(:,3))/h_opt(1)).*(abs(data_mat_p(:,3))<h_opt(1));
Kvec_h_m = (1-abs(data_mat_m(:,3))/h_opt(2)).*(abs(data_mat_m(:,3))<h_opt(2));
Kind_h_p = find(Kvec_h_p ~= 0);
Kind_h_m = find(Kvec_h_m ~= 0);
yh_p = yp(Kind_h_p,1);
yh_m = ym(Kind_h_m,1);
num = [size(yh_p,1) size(yh_m,1)];
X3h_p2 = X3h_p(Kind_h_p,1:2);
X3h_m2 = X3h_m(Kind_h_m,1:2);
Kvec_h_p2 = Kvec_h_p(Kind_h_p,1);
Kvec_h_m2 = Kvec_h_m(Kind_h_m,1);
Wh_p = diag(Kvec_h_p2);
Wh_m = diag(Kvec_h_m2);
Gamma_h_p = X3h_p2'*Wh_p*X3h_p2/n;
Gamma_h_m = X3h_m2'*Wh_m*X3h_m2/n;
invGamma_h_p = invpd(Gamma_h_p);
invGamma_h_m = invpd(Gamma_h_m);
delta_p_opt = e0'*invGamma_h_p*X3h_p2'*Wh_p*yh_p/n;
delta_m_opt = e0'*invGamma_h_m*X3h_m2'*Wh_m*yh_m/n;
knn=3;
s2h_p = (3/4)*(yh_p-nnm(yh_p,X3h_p2(:,2),knn)).^2;
s2h_m = (3/4)*(yh_m-nnm(yh_m,X3h_m2(:,2),knn)).^2;
Psi_h_p = X3h_p2'*Wh_p*diag(s2h_p)*Wh_p*X3h_p2/n;
Psi_h_m = X3h_m2'*Wh_m*diag(s2h_m)*Wh_m*X3h_m2/n;
Vh_p = (e0'*invGamma_h_p*Psi_h_p*invGamma_h_p*e0)/n;
Vh_m = (e0'*invGamma_h_m*Psi_h_m*invGamma_h_m*e0)/n;
V1 = Vh_p + Vh_m;

% Compute V2
e2 = [0 0 1]';
b2_vec_p = [1 1/b2(1) 1/b2(1)^2];
b2_vec_m = [1 1/b2(2) 1/b2(2)^2];
Hb2_p = diag(b2_vec_p);
Hb2_m = diag(b2_vec_m);
X3b2_p = data_mat_p(:,2:4)*Hb2_p;
X3b2_m = data_mat_m(:,2:4)*Hb2_m;
Kvec_b2_p = (1-abs(data_mat_p(:,3))/b2(1)).*(abs(data_mat_p(:,3))<b2(1))/b2(1);
Kvec_b2_m = (1-abs(data_mat_m(:,3))/b2(2)).*(abs(data_mat_m(:,3))<b2(2))/b2(2);
Kind_b2_p = find(Kvec_b2_p ~= 0);
Kind_b2_m = find(Kvec_b2_m ~= 0);
yb2_p = yp(Kind_b2_p,1);
yb2_m = ym(Kind_b2_m,1);
X3b2_p2 = X3b2_p(Kind_b2_p,:);
X3b2_m2 = X3b2_m(Kind_b2_m,:);
Kvec_b2_p2 = Kvec_b2_p(Kind_b2_p,1);
Kvec_b2_m2 = Kvec_b2_m(Kind_b2_m,1);
Wb2_p = diag(Kvec_b2_p2);
Wb2_m = diag(Kvec_b2_m2);
Gamma_b2_p = X3b2_p2'*Wb2_p*X3b2_p2/n;
Gamma_b2_m = X3b2_m2'*Wb2_m*X3b2_m2/n;
invGamma_b2_p = invpd(Gamma_b2_p);
invGamma_b2_m = invpd(Gamma_b2_m);
m2_p = 2*e2'*Hb2_p*invGamma_b2_p*X3b2_p2'*Wb2_p*yb2_p/n;
m2_m = 2*e2'*Hb2_m*invGamma_b2_m*X3b2_m2'*Wb2_m*yb2_m/n;
s2b2_p = (3/4)*(yb2_p-nnm(yb2_p,X3b2_p2(:,2),knn)).^2;
s2b2_m = (3/4)*(yb2_m-nnm(yb2_m,X3b2_m2(:,2),knn)).^2;
Psi_b2_p = X3b2_p2'*Wb2_p*diag(s2b2_p)*Wb2_p*X3b2_p2/n;
Psi_b2_m = X3b2_m2'*Wb2_m*diag(s2b2_m)*Wb2_m*X3b2_m2/n;
V23_b_p = 4*b2(1)^(-4)*(e2'*invGamma_b2_p*Psi_b2_p*invGamma_b2_p*e2)/n;
V23_b_m = 4*b2(2)^(-4)*(e2'*invGamma_b2_m*Psi_b2_m*invGamma_b2_m*e2)/n;
Theta2_h_p = X3h_p2'*Wh_p*X3h_p(Kind_h_p,3)/n;
Theta2_h_m = X3h_m2'*Wh_m*X3h_m(Kind_h_m,3)/n;
B012_p = e0'*H_p2*invGamma_h_p*Theta2_h_p;
B012_m = e0'*H_m2*invGamma_h_m*Theta2_h_m;
V2_p = h_opt(1)^4*V23_b_p*B012_p^2/4;
V2_m = h_opt(2)^4*V23_b_m*B012_m^2/4;
V2 = V2_p + V2_m;

% Compute C12
if h_opt(1) < b2(1)
    Kvec_b2_p3 = Kvec_b2_p(Kind_h_p,1);
    Kvec_b2_m3 = Kvec_b2_m(Kind_h_m,1);
    Wb2_p2 = diag(Kvec_b2_p3);
    Wb2_m2 = diag(Kvec_b2_m3);
    X3b2_p3 = X3b2_p(Kind_h_p,:);
    X3b2_m3 = X3b2_m(Kind_h_m,:);
    Psi13_hb2_p = X3h_p2'*Wh_p*diag(s2h_p)*Wb2_p2*X3b2_p3/n;
    Psi13_hb2_m = X3h_m2'*Wh_m*diag(s2h_m)*Wb2_m2*X3b2_m3/n;
else
    Kvec_h_p2 = Kvec_h_p(Kind_b2_p,1);
    Kvec_h_m2 = Kvec_h_m(Kind_b2_m,1);
    Wh_p2 = diag(Kvec_h_p2);
    Wh_m2 = diag(Kvec_h_m2);
    X3h_p3 = X3h_p(Kind_b2_p,1:2);
    X3h_m3 = X3h_m(Kind_b2_m,1:2);
    Psi13_hb2_p = X3h_p3'*Wh_p2*diag(s2b2_p)*Wb2_p*X3b2_p2/n;
    Psi13_hb2_m = X3h_m3'*Wh_m2*diag(s2b2_m)*Wb2_m*X3b2_m2/n;
end
D12_hb2_p = 2*b2(1)^(-2)*e0'*invGamma_h_p*Psi13_hb2_p*invGamma_b2_p*e2/n;
D12_hb2_m = 2*b2(2)^(-2)*e0'*invGamma_h_m*Psi13_hb2_m*invGamma_b2_m*e2/n;
C12_p = h_opt(1)^2*D12_hb2_p*B012_p/2;
C12_m = h_opt(2)^2*D12_hb2_m*B012_m/2;

Vrbc = V1+V2-2*C12_p-2*C12_m;
tau = delta_p_opt - delta_m_opt;
se_r = sqrt(Vrbc);
se_c = sqrt(V1);

b11_vec_opt = h_opt(1)^2*m2_p*B012_p/2;
b12_vec_opt = h_opt(2)^2*m2_m*B012_m/2;
bias_opt = b11_vec_opt-b12_vec_opt;

tval_r = (tau-bias_opt)/se_r;
pval = 2*(1-normcdf(abs(tval_r)));


% Construct bias-corrected t-value
output.h = h_opt(1);
output.num = num;
output.tau = tau;
output.se_r = se_r;
output.tval_r = tval_r;
output.pval = pval;
output.ci_r = [tau-bias_opt-1.96*se_r tau-bias_opt+1.96*se_r];
output.ci_nv = [tau-1.96*se_c tau+1.96*se_c];
output.ci_c = [tau-bias_opt-1.96*se_c tau-bias_opt+1.96*se_c];

