%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulation for RD
%
% D: DGP design
% x: evaluation point
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear;

% Simulation design
M = 10000;         % # of iteration
n = 2000;        % sample size
x_ep = 0;       % discontinuity point
D_num = 4;      % number of DGPs
methods = 8;   % number of bandwidth selection methods
kernel = 1;     % 1: triangular, 2: uniform

% Functions used in this code
%   density_est.m, density1d.m, Const_LPR_U.m, Qn_AI_h_RDD_E_R.m, srdd.m

% Get constants based on equivalent kernel for plug-in bandwidth
 C23K_U = Const_LPR_U(2,3);
 C33K_U = Const_LPR_U(3,3);
 C34K_U = Const_LPR_U(3,4);
 k_const = [-0.1 -0.1 -0.08 24/5]; % [b1 c1 c2 v]
            
% Initialization
% Checked up to this line
h_mat_1 = zeros(M,1);
h_mat_2 = zeros(M,1);
h_mat_3 = zeros(M,1);
h_mat_4 = zeros(M,2);
h_mat_5 = zeros(M,2);
h_mat_6 = zeros(M,2);

tau_bias_mat_1 = zeros(M,1);
tau_bias_mat_2 = zeros(M,1);
tau_bias_mat_3 = zeros(M,1);
tau_bias_mat_4 = zeros(M,1);
tau_bias_mat_5 = zeros(M,1);
tau_bias_mat_6 = zeros(M,1);

delta_bias_mat_1 = zeros(M,2);
delta_bias_mat_2 = zeros(M,2);
delta_bias_mat_3 = zeros(M,2);
delta_bias_mat_4 = zeros(M,2);

all_result_mat = zeros(2*D_num,methods);

Coverage_mat = zeros(D_num,methods);
Length_mat = zeros(D_num,methods);



tic

%
% Begin Monte Carlo
%

    
% D_single = 1;
% for D=D_single:D_single
% for D=1:D_num
for D=1:4

rng(10);

count = zeros(1,methods);
Length_all = zeros(M,methods);

if D==1
    load('SRD_h_D1_16_n2k'); 
    h_mat_aip = h_mat(:,1);
    h_mat_aim = h_mat(:,5);
    h_ai = [h_mat_aip h_mat_aim];
    h_mat_cct = h_mat(:,4);
    h_cct = [h_mat_cct h_mat_cct];
        
    load('SRD_h23_D1_16_n2k');
    h2 = h23_mat;
    
    load('SRD_h3_D1_16_n2k');
    h3 = h3_mat;
    
elseif D==2
    load('SRD_h_D2_16_n2k'); 
    h_mat_aip = h_mat(:,1);
    h_mat_aim = h_mat(:,5);
    h_ai = [h_mat_aip h_mat_aim];
    h_mat_cct = h_mat(:,4);
    h_cct = [h_mat_cct h_mat_cct];
        
    load('SRD_h23_D2_16_n2k');
    h2 = h23_mat;
    
    load('SRD_h3_D2_16_n2k');
    h3 = h3_mat;
    
elseif D==3
    load('SRD_h_D3_16_n2k'); 
    h_mat_aip = h_mat(:,1);
    h_mat_aim = h_mat(:,5);
    h_ai = [h_mat_aip h_mat_aim];
    h_mat_cct = h_mat(:,4);
    h_cct = [h_mat_cct h_mat_cct];
        
    load('SRD_h23_D3_16_n2k');
    h2 = h23_mat;
    
    load('SRD_h3_D3_16_n2k');
    h3 = h3_mat;
    
else
    load('SRD_h_D4_16_n2k'); 
    h_mat_aip = h_mat(:,1);
    h_mat_aim = h_mat(:,5);
    h_ai = [h_mat_aip h_mat_aim];
    h_mat_cct = h_mat(:,4);
    h_cct = [h_mat_cct h_mat_cct];
        
    load('SRD_h23_D4_16_n2k');
    h2 = h23_mat;
    
    load('SRD_h3_D4_16_n2k');
    h3 = h3_mat;
    
end


for ite = 1:M
    % Data generation
    [Y,X,Tmat] = data_RDD(n,D);
    true_f  = Tmat(1,1);
    true_f1 = Tmat(1,2);
    true_sigma_p = Tmat(1,3);
    true_sigma_m = Tmat(1,4);
    true_tau  = Tmat(1,5);
    true_m21 = Tmat(1,6);
    true_m22 = Tmat(1,7);
    true_m31 = Tmat(1,8);
    true_m32 = Tmat(1,9);
    true_delta_p = Tmat(1,10);
    true_delta_m = Tmat(1,11);
    true_m41 = Tmat(1,12);
    true_m42 = Tmat(1,13);
    true_m51 = Tmat(1,14);
    true_m52 = Tmat(1,15);
    m2_true = [true_m21 true_m22];
    m3_true = [true_m31 true_m32];
    delta_true = [true_delta_p true_delta_m];

 
% MMSE

    outputai = rddci_mmse_v01b(X,Y,0,h_ai(ite,:),h2(ite,:));
    
    % 1. Non-Valid
    ciai_nv = outputai.ci_nv;
    if true_tau>ciai_nv(1)
        if true_tau<ciai_nv(2)
            count(1,1)= count(1,1) + 1;
        else
        end
    else
    end
    Length_all(ite,1) = ciai_nv(2)-ciai_nv(1);
    
    % 3. Conventional
    ciai_c = outputai.ci_c;
    if true_tau>ciai_c(1)
        if true_tau<ciai_c(2)
            count(1,3)= count(1,3) + 1;
        else
        end
    else
    end
    Length_all(ite,3) = ciai_c(2)-ciai_c(1); 
    
    % 5. Undersmoothing
    ciai_us = outputai.ci_us;
    if true_tau>ciai_us(1)
        if true_tau<ciai_us(2)
            count(1,5)= count(1,5) + 1;
        else
        end
    else
    end
    Length_all(ite,5) = ciai_us(2)-ciai_us(1); 
    
    % 6. Robust
    ciai_r = outputai.ci_r;
    if true_tau>ciai_r(1)
        if true_tau<ciai_r(2)
            count(1,6)= count(1,6) + 1;
        else
        end
    else
    end
    Length_all(ite,6) = ciai_r(2)-ciai_r(1); 
    
% CCT

% 2. Non-Valid

    outputcicct = rddci_cct_b(X,Y,0,[h_cct(ite) h_cct(ite)],[h_cct(ite) h_cct(ite)]);
    cicct_nv = outputcicct.ci_nv;
    if true_tau>cicct_nv(1)
        if true_tau<cicct_nv(2)
            count(1,2) = count(1,2) + 1;
        else
        end
    else
    end
    Length_all(ite,2) = cicct_nv(2)-cicct_nv(1); 
    
% 4. Conventional

    outputcicct = rddci_cct_b(X,Y,0,[h_cct(ite) h_cct(ite)],[h_cct(ite) h_cct(ite)]);
    cicct_c = outputcicct.ci_c;
    if true_tau>cicct_c(1)
        if true_tau<cicct_c(2)
            count(1,4) = count(1,4) + 1;
        else
        end
    else
    end
    Length_all(ite,4) = cicct_c(2)-cicct_c(1); 
    
  
% 7. Robust
      
    outputcicct = rddci_cct(X,Y,0,[h_cct(ite) h_cct(ite)],[h_cct(ite) h_cct(ite)]);
    cicct_r = outputcicct.ci_r;
    if true_tau>cicct_r(1)
        if true_tau<cicct_r(2)
            count(1,7) = count(1,7) + 1;
        else
        end
    else
    end
    Length_all(ite,7) = cicct_r(2)-cicct_r(1); 
 
%    
% 8. CCF
%      
    h_ccf = h_cct*n^(-1/20);
    outputciccf = rddci_cct(X,Y,0,[h_ccf(ite) h_ccf(ite)],[h_ccf(ite) h_ccf(ite)]);
    ciccf = outputciccf.ci_r;
    if true_tau>ciccf(1)
        if true_tau<ciccf(2)
            count(1,8) = count(1,8) + 1;
        else
        end
    else
    end
    Length_all(ite,8) = ciccf(2)-ciccf(1);

    
    
% end of M iteration
end




Coverage_mat(D,:) = count/M;
Length_mat(D,:) = mean(Length_all);
all_result_mat(2*(D-1)+1,:) = Coverage_mat(D,:);
all_result_mat(2*(D-1)+2,:) = Length_mat(D,:);



D

% end of D loop
end



all_result_mat

toc
beep
